from django.urls import path
from . import models, views
from netbox.views.generic import ObjectChangeLogView

urlpatterns = (

    path('documents/', views.DocumentListView.as_view(), name='document_list'),
    path('documents/add/', views.DocumentEditView.as_view(), name='document_add'),
    path('documents/addlink/', views.DocumentCreateView.as_view(), name='document_add_link'),
    path('documents/<int:pk>/', views.DocumentView.as_view(), name='document'),
    path('documents/<int:pk>/edit/', views.DocumentEditView.as_view(), name='document_edit'),
    path('documents/<int:pk>/delete/', views.DocumentDeleteView.as_view(), name='document_delete'),
    path('documents/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='document_changelog', kwargs={
        'model': models.Document
    }),

    path('types/', views.DocumentTypeListView.as_view(), name='documenttype_list'),
    path('types/add/', views.DocumentTypeEditView.as_view(), name='documenttype_add'),
    path('types/<int:pk>/', views.DocumentTypeView.as_view(), name='documenttype'),
    path('types/<int:pk>/edit/', views.DocumentTypeEditView.as_view(), name='documenttype_edit'),
    path('types/<int:pk>/delete/', views.DocumentTypeDeleteView.as_view(), name='documenttype_delete'),
    path('types/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='documenttype_changelog', kwargs={
        'model': models.DocumentType
    }),

    path('link/add/', views.DocumentLinkEditView.as_view(), name='documentlink_add'),
    path('link/<int:pk>/delete', views.DocumentLinkDeleteView.as_view(), name='documentlink_delete'),

)



from extras.plugins import PluginMenuItem, PluginMenu, PluginMenuButton
from utilities.choices import ButtonColorChoices
from django.conf import settings

menuitem = [
    PluginMenuItem(
        link='plugins:netbox_ged:document_list',
        link_text='Documents',
        buttons=[PluginMenuButton(
            link='plugins:netbox_ged:document_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN
        )]
    ),
    PluginMenuItem(
        link='plugins:netbox_ged:documenttype_list',
        link_text='Document Types',
        buttons=[PluginMenuButton(
            link='plugins:netbox_ged:documenttype_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN
        )]
    ),
]

# If we are using NB 3.4.0+ display the new top level navigation option
if settings.VERSION >= '3.4.0':

    menu = PluginMenu(
        label='Documents',
        groups=(
            ('Document Storage', menuitem),
        ),
        icon_class='mdi mdi-file-document-multiple'
    )

else:

    # Fall back to pre 3.4 navigation option
    menu_items = menuitem



from django.core.exceptions import ValidationError
from django.db import models
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey

from netbox.models import NetBoxModel
from utilities.fields import ColorField

from .utils import file_upload


class DocumentType(NetBoxModel):
    name = models.CharField(
        max_length=100,
        unique=True,
        help_text='Name of the document type'
    )

    color = ColorField()

    class Meta:
        ordering = ['name']
        verbose_name_plural = "Document Types"
        verbose_name = "Document Type"

    def __str__(self):
        return self.name

    # Needed for linkify in tables
    def get_absolute_url(self):
        return reverse('plugins:netbox_ged:documenttype', args=[self.pk])


class Document(NetBoxModel):
    name = models.CharField(
        max_length=100,
        blank=True,
        help_text='(Optional) Specify a name to display for this document. If no name is specified, the filename or url will be used.'
    )

    document = models.FileField(
        upload_to=file_upload,
        blank=True
    )

    document_type = models.ForeignKey(
        to=DocumentType,
        on_delete=models.PROTECT,
        related_name='documents',
    )

    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('-created', 'name')
        verbose_name_plural = "Documents"
        verbose_name = "Document"

    def get_document_type_color(self):
        return self.document_type.color

    @property
    def size(self):
        """
        Wrapper around `document.size` to suppress an OSError in case the file is inaccessible.     Also opportunistically
        catch other exceptions that we know other storage back-ends to throw.
        """
        expected_exceptions = [OSError]

        try:
            from botocore.exceptions import ClientError
            expected_exceptions.append(ClientError)
        except ImportError:
            pass

        try:
            return self.document.size
        except:
            return None

    @property
    def filename(self):
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def __str__(self):
        if self.name:
            return self.name

        elif self.document:
            filename = self.document.name.rsplit('/', 1)[-1]
            return filename.split('_', 1)[1]

        else:
            return ""

    def get_absolute_url(self):
        return reverse('plugins:netbox_ged:document', args=[self.pk])

    def clean(self):
        super().clean()

        # Must have an uploaded document
        if not self.document:
            raise ValidationError("A document must contain an uploaded file")

    def delete(self, *args, **kwargs):
        _name = self.document.name

        # Delete file from disk
        super().delete(*args, **kwargs)
        self.document.delete(save=False)

        # Restore the name of the document as it's re-used in the notifications later
        self.document.name = _name


class DocumentLink(NetBoxModel):
    document = models.ForeignKey(
        to=Document,
        on_delete=models.PROTECT,
        related_name='relative_objects',
    )
    content_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.PROTECT,
        related_name='documents',
    )
    object_id = models.PositiveIntegerField()
    object_content = GenericForeignKey(
        ct_field='content_type',
        fk_field='object_id'
    )



from netbox.search import SearchIndex
from .models import Document
from django.conf import settings

# If we run NB 3.4+ register search indexes 
if settings.VERSION >= '3.4.0':
    class DocumentIndex(SearchIndex):
        model = Document
        fields = (
            ("name", 100),
            ("document_type__name", 500),
        )


    # Register indexes
    indexes = [DocumentIndex]



from netbox.api.viewsets import NetBoxModelViewSet

from .serializers import DocumentSerializer, DocumentTypeSerializer
from ..filtersets import DocumentTypeFilterSet, DocumentFilterSet
from ..models import DocumentType, Document


class DocumentViewSet(NetBoxModelViewSet):
    queryset = Document.objects.prefetch_related('tags')
    serializer_class = DocumentSerializer
    filterset_class = DocumentFilterSet


class DocumentTypeViewSet(NetBoxModelViewSet):
    queryset = DocumentType.objects.prefetch_related('tags')
    serializer_class = DocumentTypeSerializer
    filterset_class = DocumentTypeFilterSet



from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer, WritableNestedSerializer
from ..models import Document, DocumentType, DocumentLink


class DocumentLinkSerializer(WritableNestedSerializer):
    class Meta:
        model = DocumentLink
        fields = ['id', 'document', 'content_type', 'object_id']

# Site Document Serializer
class DocumentSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_ged-api:document-detail'
    )
    relative_objects = DocumentLinkSerializer(many=True, read_only=True)

    class Meta:
        model = Document
        fields = (
            'id', 'url', 'display', 'relative_objects', 'name', 'document', 'document_type', 'filename', 'comments', 'tags',
            'custom_fields', 'created', 'last_updated',
        )


# Site Document Type Serializer
class DocumentTypeSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_ged-api:documenttype-detail')

    class Meta:
        model = DocumentType
        fields = (
            'id', 'url', 'display', 'name', 'color', 'tags', 'custom_fields', 'created',
            'last_updated',
        )




from django.contrib.contenttypes.models import ContentType
from django.forms import CharField, IntegerField
from extras.utils import FeatureQuery
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms import TagFilterField, CommentField, DynamicModelMultipleChoiceField, DynamicModelChoiceField, \
    ContentTypeChoiceField
from .models import Document, DocumentType, DocumentLink


# Document Form & Filter Form
class DocumentForm(NetBoxModelForm):
    comments = CommentField()
    document_type = DynamicModelChoiceField(
        queryset=DocumentType.objects.all(),
        required=True,
    )

    class Meta:
        model = Document
        fields = ('name', 'document', 'document_type', 'comments', 'tags')


class DocumentFilterForm(NetBoxModelFilterSetForm):
    model = Document

    name = CharField(
        required=False
    )

    document_type_id = DynamicModelMultipleChoiceField(
        queryset=DocumentType.objects.all(),
        required=False,
    )

    tag = TagFilterField(model)


# Document Type Form & Filter Form
class DocumentTypeForm(NetBoxModelForm):
    class Meta:
        model = DocumentType
        fields = ('name', 'color')


class DocumentTypeFilterForm(NetBoxModelFilterSetForm):
    model = DocumentType

    name = CharField(
        required=False
    )

    slug = CharField(
        required=False
    )


class DocumentLinkForm(NetBoxModelForm):
    document = DynamicModelChoiceField(
        queryset=Document.objects.all(),
        required=True,
        label='Document',
        help_text='Select the document to link to this object.'
    )
    content_type = ContentTypeChoiceField(
        queryset=ContentType.objects.all(),
        limit_choices_to=FeatureQuery('webhooks'),
        required=False,
        help_text="Type of the related object"
    )

    class Meta:
        model = DocumentLink
        fields = ('document', 'content_type', 'object_id')


class DocumentCreateForm(DocumentForm):
    content_type = ContentTypeChoiceField(
        queryset=ContentType.objects.all(),
        limit_choices_to=FeatureQuery('webhooks'),
        required=True,
        help_text="Type of the related object"
    )
    object_id = IntegerField(
        required=True,
        help_text="ID of the related object"
    )

    def save(self, *args, **kwargs):
        document = super().save(*args, **kwargs)
        DocumentLink.objects.create(
            document=document,
            content_type=self.cleaned_data['content_type'],
            object_id=self.cleaned_data['object_id']
        )
        return document



from extras.plugins import PluginConfig


class NetboxGED(PluginConfig):
    name = 'netbox_ged'
    verbose_name = 'Document Storage'
    description = 'Manage documents in Netbox'
    version = '0.2.0'
    author = 'Pelt10'
    author_email = 'contact@lethiec.me'
    min_version = '3.2.0'
    base_url = 'documents'


config = NetboxGED



import django_tables2 as tables

from netbox.tables import NetBoxTable, columns
from .models import Document, DocumentType

DOCUMENT_LINK = """
<a href="{% url 'plugins:netbox_ged:document' pk=record.pk %}">{% firstof record.name record.filename %}</a> (<a href="{{record.document.url}}" target="_blank">View Document</a>)
"""


class DocumentTable(NetBoxTable):
    name = tables.TemplateColumn(template_code=DOCUMENT_LINK)
    document_type = columns.ColoredLabelColumn()

    tags = columns.TagColumn(
        url_name='plugins:netbox_ged:document_list'
    )

    class Meta(NetBoxTable.Meta):
        model = Document
        fields = ('pk', 'id', 'name', 'document_type',  'size', 'filename', 'comments', 'actions', 'created', 'last_updated', 'tags')
        default_columns = ('name', 'document_type', 'tags')


class DocumentTypeTable(NetBoxTable):
    name = tables.Column(
        linkify=True,
    )
    color = columns.ColorColumn()

    class Meta(NetBoxTable.Meta):
        model = DocumentType
        fields = ('pk', 'id', 'name', 'color')
        default_columns = ('name', 'color')



from netbox.api.routers import NetBoxRouter

from netbox_ged.api.views import DocumentViewSet, DocumentTypeViewSet

app_name = 'netbox_ged'

router = NetBoxRouter()
router.register('documents', DocumentViewSet)
router.register('documenttype', DocumentTypeViewSet)

urlpatterns = router.urls


from netbox.filtersets import NetBoxModelFilterSet
from .models import Document, DocumentType
from django.db.models import Q


class DocumentFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = Document
        fields = ('id', 'name', 'document_type')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(document__icontains=value)
        )


class DocumentTypeFilterSet(NetBoxModelFilterSet):

        class Meta:
            model = DocumentType
            fields = ('id', 'name')

        def search(self, queryset, name, value):
            if not value.strip():
                return queryset
            return queryset.filter(
                Q(name__icontains=value) |
                Q(slug__icontains=value)
            )


from netbox.views import generic
from .filtersets import DocumentTypeFilterSet, DocumentFilterSet
from .forms import DocumentTypeFilterForm, DocumentTypeForm, DocumentForm, DocumentFilterForm, DocumentLinkForm, \
    DocumentCreateForm
from .models import Document, DocumentType, DocumentLink
from .tables import DocumentTypeTable, DocumentTable


# Document
class DocumentView(generic.ObjectView):
    queryset = Document.objects.all()


class DocumentListView(generic.ObjectListView):
    queryset = Document.objects.all()
    table = DocumentTable
    filterset = DocumentFilterSet
    filterset_form = DocumentFilterForm


class DocumentEditView(generic.ObjectEditView):
    queryset = Document.objects.all()
    form = DocumentForm


class DocumentDeleteView(generic.ObjectDeleteView):
    queryset = Document.objects.all()


# DocumentType
class DocumentTypeView(generic.ObjectView):
    queryset = DocumentType.objects.all()


class DocumentTypeListView(generic.ObjectListView):
    queryset = DocumentType.objects.all()
    table = DocumentTypeTable
    filterset = DocumentTypeFilterSet
    filterset_form = DocumentTypeFilterForm


class DocumentTypeEditView(generic.ObjectEditView):
    queryset = DocumentType.objects.all()
    form = DocumentTypeForm


class DocumentTypeDeleteView(generic.ObjectDeleteView):
    queryset = DocumentType.objects.all()


class DocumentLinkEditView(generic.ObjectEditView):
    queryset = DocumentLink.objects.all()
    form = DocumentLinkForm


class DocumentLinkDeleteView(generic.ObjectDeleteView):
    queryset = DocumentLink.objects.all()


# View to create a new document and document link in one step
class DocumentCreateView(generic.ObjectEditView):
    queryset = Document.objects.all()
    form = DocumentCreateForm

    def get_initial(self):
        initial = super().get_initial()
        initial['document_type'] = DocumentType.objects.first()
        return initial

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['form'].fields['document_type'].queryset = DocumentType.objects.all()
        return context



from django.contrib.contenttypes.models import ContentType

from extras.plugins import PluginTemplateExtension
from extras.utils import FeatureQuery

from netbox_ged.models import Document


class DocumentTemplateExtension(PluginTemplateExtension):

    def left_page(self):
        content_type = ContentType.objects.get(
            app_label=self.context['object']._meta.app_label,
            model=self.context['object']._meta.model_name
        )

        documents = Document.objects.filter(
                relative_objects__content_type=content_type.id,
                relative_objects__object_id=self.context['object'].pk
        )

        for document in documents:
            document.link = document.relative_objects.get(
                content_type=content_type.id,
                object_id=self.context['object'].pk,
                document=document
            ).id

        return self.render('netbox_ged/document_include.html', extra_context={
            'documents': documents,
            'content_type': content_type,
        })


template_extensions = []
for ct in ContentType.objects.filter(FeatureQuery('webhooks').get_query()):
    class_name = ct.model_class().__name__ + 'DocumentList'
    model_name = "%s.%s" % (ct.app_label, ct.model)
    template_extensions.append(type(class_name, (DocumentTemplateExtension,), {'model': model_name}))




# Generated by Django 4.1.5 on 2023-02-07 22:14

from django.db import migrations, models
import django.db.models.deletion
import netbox_ged.utils
import taggit.managers
import utilities.fields
import utilities.json


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('extras', '0084_staging'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(blank=True, max_length=100)),
                ('document', models.FileField(blank=True, upload_to=netbox_ged.utils.file_upload)),
                ('comments', models.TextField(blank=True)),
            ],
            options={
                'verbose_name': 'Document',
                'verbose_name_plural': 'Documents',
                'ordering': ('-created', 'name'),
            },
        ),
        migrations.CreateModel(
            name='DocumentType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('color', utilities.fields.ColorField(max_length=6)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Document Type',
                'verbose_name_plural': 'Document Types',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='DocumentLink',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='documents', to='contenttypes.contenttype')),
                ('document', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='relative_objects', to='netbox_ged.document')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='document',
            name='document_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='documents', to='netbox_ged.documenttype'),
        ),
        migrations.AddField(
            model_name='document',
            name='tags',
            field=taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag'),
        ),
    ]
